searchData={"items":[{"type":"module","title":"Hammer.Mnesia","doc":"This backend uses the `:mnesia`.\n\n    defmodule MyApp.RateLimit do\n      # the default table is the current's module name\n      use Hammer, backend: Hammer.Mnesia, table: __MODULE__\n    end\n\n    # attempts to create a distributed in-memory table and run the clean up every 10 minutes\n    MyApp.RateLimit.start_link(clean_period: :timer.minutes(10))\n\n    {:allow, _count} = MyApp.RateLimit.hit(key, scale, limit)","ref":"Hammer.Mnesia.html"},{"type":"function","title":"Hammer.Mnesia.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Hammer.Mnesia.html#child_spec/1"},{"type":"type","title":"Hammer.Mnesia.mnesia_option/0","doc":"","ref":"Hammer.Mnesia.html#t:mnesia_option/0"},{"type":"type","title":"Hammer.Mnesia.mnesia_options/0","doc":"","ref":"Hammer.Mnesia.html#t:mnesia_options/0"},{"type":"function","title":"Hammer.Mnesia.now/0","doc":"","ref":"Hammer.Mnesia.html#now/0"},{"type":"function","title":"Hammer.Mnesia.start_link/1","doc":"Starts the process that creates and cleans the ETS table.\n\nAccepts the following options:\n  - `:clean_period` for how often to perform garbage collection\n  - optional mnesia options from `:mnesia.create_table/2`\n  - `:key_older_than` - How old a key can be before it is removed from the table (in milliseconds). Defaults to 10 minutes.\n  - `:algorithm` - The rate limiting algorithm to use. Can be `:fixed_window`, `:token_bucket`, or `:leaky_bucket`. Defaults to `:fixed_window`.\n  - optional `:debug`, `:spawn_opts`, and `:hibernate_after` `GenServer.options()`","ref":"Hammer.Mnesia.html#start_link/1"},{"type":"module","title":"Hammer.Mnesia.FixWindow","doc":"This module implements the Fix Window algorithm for the [Mnesia](https://hex.pm/packages/mnesia) backend.\n\nThe fixed window algorithm works by dividing time into fixed intervals or \"windows\"\nof a specified duration (scale). Each window tracks request counts independently.\n\nFor example, with a 60 second window:\n- Window 1: 0-60 seconds\n- Window 2: 60-120 seconds\n- And so on...\n\nThe algorithm:\n1. When a request comes in, we:\n   - Calculate which window it belongs to based on current time\n   - Increment the counter for that window\n   - Store expiration time as end of window\n2. To check if rate limit is exceeded:\n   - If count <= limit: allow request\n   - If count > limit: deny and return time until window expires\n3. Old windows are automatically cleaned up after expiration\n\nThis provides simple rate limiting but has edge cases where a burst of requests\nspanning a window boundary could allow up to 2x the limit in a short period.\nFor more precise limiting, consider using the sliding window algorithm instead.\n\nThe fixed window algorithm is a good choice when:\n\n- You need simple, predictable rate limiting with clear time boundaries\n- The exact precision of the rate limit is not critical\n- You want efficient implementation with minimal storage overhead\n- Your use case can tolerate potential bursts at window boundaries\n\nCommon use cases include:\n\n- Basic API rate limiting where occasional bursts are acceptable\n- Protecting backend services from excessive load\n- Implementing fair usage policies\n- Scenarios where clear time-based quotas are desired (e.g. \"100 requests per minute\")\n\nThe main tradeoff is that requests near window boundaries can allow up to 2x the\nintended limit in a short period. For example with a limit of 100 per minute:\n- 100 requests at 11:59:59\n- Another 100 requests at 12:00:01\n\nThis results in 200 requests in 2 seconds, while still being within limits.\nIf this behavior is problematic, consider using the sliding window algorithm instead.\n\nThe fixed window algorithm supports the following options:\n\n- `:clean_period` - How often to run the cleanup process (in milliseconds)\n  Defaults to 1 minute. The cleanup process removes expired window entries.\n\nExample configuration:\n\n    MyApp.RateLimit.start_link(\n      clean_period: :timer.minutes(5),\n    )\n\nThis would run cleanup every 5 minutes and clean up old windows.","ref":"Hammer.Mnesia.FixWindow.html"},{"type":"function","title":"Hammer.Mnesia.FixWindow.clean/1","doc":"","ref":"Hammer.Mnesia.FixWindow.html#clean/1"},{"type":"function","title":"Hammer.Mnesia.FixWindow.full_key/2","doc":"","ref":"Hammer.Mnesia.FixWindow.html#full_key/2"},{"type":"function","title":"Hammer.Mnesia.FixWindow.full_key/3","doc":"","ref":"Hammer.Mnesia.FixWindow.html#full_key/3"},{"type":"function","title":"Hammer.Mnesia.FixWindow.mnesia_opts/0","doc":"","ref":"Hammer.Mnesia.FixWindow.html#mnesia_opts/0"},{"type":"module","title":"Hammer.Mnesia.LeakyBucket","doc":"This module implements the Leaky Bucket algorithm.\n\nThe leaky bucket algorithm works by modeling a bucket that:\n- Fills up with requests at the input rate\n- \"Leaks\" requests at a constant rate\n- Has a maximum capacity (the bucket size)\n\nFor example, with a leak rate of 10 requests/second and bucket size of 100:\n- Requests add to the bucket's current level\n- The bucket leaks 10 requests per second steadily\n- If bucket reaches capacity (100), new requests are denied\n- Once bucket level drops, new requests are allowed again\n\nThe algorithm:\n1. When a request comes in, we:\n   - Calculate how much has leaked since last request\n   - Subtract leaked amount from current bucket level\n   - Try to add new request to bucket\n   - Store new bucket level and timestamp\n2. To check if rate limit is exceeded:\n   - If new bucket level <= capacity: allow request\n   - If new bucket level > capacity: deny and return time until enough leaks\n3. Old entries are automatically cleaned up after expiration\n\nThis provides smooth rate limiting with ability to handle bursts up to bucket size.\nThe leaky bucket is a good choice when:\n\n- You need to enforce a constant processing rate\n- Want to allow temporary bursts within bucket capacity\n- Need to smooth out traffic spikes\n- Want to prevent resource exhaustion\n\nCommon use cases include:\n\n- API rate limiting needing consistent throughput\n- Network traffic shaping\n- Service protection from sudden load spikes\n- Queue processing rate control\n- Scenarios needing both burst tolerance and steady-state limits\n\nThe main advantages are:\n- Smooth, predictable output rate\n- Configurable burst tolerance\n- Natural queueing behavior\n\nThe tradeoffs are:\n- More complex implementation than fixed windows\n- Need to track last request time and current bucket level\n- May need tuning of bucket size and leak rate parameters\n\nFor example, with 100 requests/sec limit and 500 bucket size:\n- Can handle bursts of up to 500 requests\n- But long-term average rate won't exceed 100/sec\n- Provides smoother traffic than fixed windows\n\nThe leaky bucket algorithm supports the following options:\n\n- `:clean_period` - How often to run the cleanup process (in milliseconds)\n  Defaults to 1 minute. The cleanup process removes expired bucket entries.\n\n- `:key_older_than` - Optional maximum age for bucket entries (in milliseconds)\n  If set, entries older than this will be removed during cleanup.\n  This helps prevent memory growth from abandoned buckets.\n\nExample configuration:\n\n    MyApp.RateLimit.start_link(\n      clean_period: :timer.minutes(5),\n      key_older_than: :timer.minutes(10)\n    )\n\nThis would run cleanup every 5 minutes and remove buckets not used in 24 hours.","ref":"Hammer.Mnesia.LeakyBucket.html"},{"type":"function","title":"Hammer.Mnesia.LeakyBucket.clean/1","doc":"","ref":"Hammer.Mnesia.LeakyBucket.html#clean/1"},{"type":"function","title":"Hammer.Mnesia.LeakyBucket.get/2","doc":"Returns the current level of the bucket for a given key.","ref":"Hammer.Mnesia.LeakyBucket.html#get/2"},{"type":"function","title":"Hammer.Mnesia.LeakyBucket.hit/5","doc":"Checks if a key is allowed to perform an action, and increment the counter by the given amount.","ref":"Hammer.Mnesia.LeakyBucket.html#hit/5"},{"type":"module","title":"Hammer.Mnesia.TokenBucket","doc":"This module implements the Token Bucket algorithm.\nThe token bucket algorithm works by modeling a bucket that:\n- Fills with tokens at a constant rate (the refill rate)\n- Has a maximum capacity of tokens (the bucket size)\n- Each request consumes one or more tokens\n- If there are enough tokens, the request is allowed\n- If not enough tokens, the request is denied\n\nFor example, with a refill rate of 10 tokens/second and bucket size of 100:\n- Tokens are added at 10 per second up to max of 100\n- Each request needs tokens to proceed\n- If bucket has enough tokens, request allowed and tokens consumed\n- If not enough tokens, request denied until bucket refills\n\nThe algorithm:\n1. When a request comes in, we:\n   - Calculate tokens added since last request based on time elapsed\n   - Add new tokens to bucket (up to max capacity)\n   - Try to consume tokens for the request\n   - Store new token count and timestamp\n2. To check if rate limit is exceeded:\n   - If enough tokens: allow request and consume tokens\n   - If not enough: deny and return time until enough tokens refill\n3. Old entries are automatically cleaned up after expiration\n\nThis provides smooth rate limiting with ability to handle bursts up to bucket size.\nThe token bucket is a good choice when:\n\n- You need to allow temporary bursts of traffic\n- Want to enforce an average rate limit\n- Need to support different costs for different operations\n- Want to avoid the sharp edges of fixed windows\n\nCommon use cases include:\n\n- API rate limiting with burst tolerance\n- Network traffic shaping\n- Resource allocation control\n- Gaming systems with \"energy\" mechanics\n- Scenarios needing flexible rate limits\n\nThe main advantages are:\n- Natural handling of bursts\n- Flexible token costs for different operations\n- Smooth rate limiting behavior\n- Simple to reason about\n\nThe tradeoffs are:\n- Need to track token count and last update time\n- May need tuning of bucket size and refill rate\n- More complex than fixed windows\n\nFor example with 100 tokens/minute limit and 500 bucket size:\n- Can handle bursts using saved up tokens\n- Automatically smooths out over time\n- Different operations can cost different amounts\n- More flexible than fixed request counts\n\nThe token bucket algorithm supports the following options:\n\n- `:clean_period` - How often to run the cleanup process (in milliseconds)\n  Defaults to 1 minute. The cleanup process removes expired bucket entries.\n\n- `:key_older_than` - Optional maximum age for bucket entries (in milliseconds)\n  If set, entries older than this will be removed during cleanup.\n  This helps prevent memory growth from abandoned buckets.\n\nExample configuration:\n\n    MyApp.RateLimit.start_link(\n      clean_period: :timer.minutes(5),\n      key_older_than: :timer.minutes(10)\n    )\n\nThis would run cleanup every 5 minutes and remove buckets not used in 24 hours.","ref":"Hammer.Mnesia.TokenBucket.html"},{"type":"function","title":"Hammer.Mnesia.TokenBucket.clean/1","doc":"","ref":"Hammer.Mnesia.TokenBucket.html#clean/1"},{"type":"function","title":"Hammer.Mnesia.TokenBucket.get/2","doc":"Returns the current level of the bucket for a given key.","ref":"Hammer.Mnesia.TokenBucket.html#get/2"},{"type":"extras","title":"Hammer.Mnesia","doc":"# Hammer.Mnesia\n\n[![Build Status](https://github.com/ExHammer/hammer-backend-mnesia/actions/workflows/ci.yml/badge.svg)](https://github.com/ExHammer/hammer-backend-mnesia/actions/workflows/ci.yml)\n[![Hex.pm](https://img.shields.io/hexpm/v/hammer_backend_mnesia.svg)](https://hex.pm/packages/hammer_backend_mnesia)\n[![Documentation](https://img.shields.io/badge/documentation-gray)](https://hexdocs.pm/hammer_backend_mnesia)\n[![Total Download](https://img.shields.io/hexpm/dt/hammer_backend_mnesia.svg)](https://hex.pm/packages/hammer_backend_mnesia)\n[![License](https://img.shields.io/hexpm/l/hammer_backend_mnesia.svg)](https://github.com/ExHammer/hammer-backend-mnesia/blob/master/LICENSE.md)\n\n\n---\n\n> [!NOTE]\n>\n> This README is for the unreleased master branch, please reference the [official documentation on hexdocs](https://hexdocs.pm/hammer_backend_mnesia/) for the latest stable release.\n\n---\n\nA Mnesia backend for the [Hammer](https://github.com/ExHammer/hammer) rate-limiter.\n\nThis package is available in beta. If you have any problems, please open an issue.\n\n> [!TIP]\n> Consider using `Hammer.ETS` with counter increments broadcasted via [Phoenix PubSub](https://hexdocs.pm/phoenix_pubsub/Phoenix.PubSub.html) instead.","ref":"readme.html"},{"type":"extras","title":"Installation - Hammer.Mnesia","doc":"The package can be installed by adding `hammer_backend_mnesia` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:hammer_backend_mnesia, \"~> 0.7.0\"},\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Usage - Hammer.Mnesia","doc":"1. Define the rate limiter using `Hammer.Mnesia` backend:\n\n    ```elixir\n    defmodule MyApp.RateLimit do\n      use Hammer, backend: Hammer.Mnesia\n    end\n    ```\n\n2. Add the rate limiter to your supervision tree:\n\n    ```elixir\n    children = [\n      # ...\n      {MyApp.RateLimit, clean_period: :timer.minutes(1)}\n      # ...\n    ]\n    ```\n\n3. And that's it, calls to the rate limiter will use Mnesia to store the counters.\n\n    ```elixir\n    case MyApp.RateLimit.hit(key, _scale = :timer.minutes(1), _limit = 100) do\n      {:allow, _count} -> :ok\n      {:deny, retry_after} -> {:error, :rate_limit, \"retry after #{retry_after}ms\"}\n    end\n    ```","ref":"readme.html#usage"},{"type":"extras","title":"Documentation - Hammer.Mnesia","doc":"On hexdocs: [https://hexdocs.pm/hammer_backend_mnesia/](https://hexdocs.pm/hammer_backend_mnesia/)","ref":"readme.html#documentation"},{"type":"extras","title":"Getting Help - Hammer.Mnesia","doc":"If you're having trouble, either open an issue on this repo, or reach out to the\nmaintainers ([@shanekilkelly](https://twitter.com/shanekilkelly)) on Twitter.","ref":"readme.html#getting-help"},{"type":"extras","title":"CHANGELOG","doc":"","ref":"changelog.html"},{"type":"extras","title":"0.7.0-rc.0 - 2024-12-06 - CHANGELOG","doc":"","ref":"changelog.html#0-7-0-rc-0-2024-12-06"},{"type":"extras","title":"Changed - CHANGELOG","doc":"- adapt to the new Hammer API\n- Added `:algorithm` option to the Atomic backend with support for:\n  - `:fix_window` (default) - Fixed time window rate limiting\n  - `:leaky_bucket` - Constant rate limiting with burst capacity\n  - `:token_bucket` - Token-based rate limiting with burst capacity\n- use [`:mnesia.dirty_update_counter/3`](https://www.erlang.org/doc/apps/mnesia/mnesia.html#dirty_update_counter/3)\n- automatically create an in-memory table (no schema needed)\n- listen for cluster changes and replicate the in-memory table\n- require Elixir 1.14+ and OTP 25+","ref":"changelog.html#changed"},{"type":"extras","title":"0.6.1 - 2024-03-29 - CHANGELOG","doc":"","ref":"changelog.html#0-6-1-2024-03-29"},{"type":"extras","title":"Changed - CHANGELOG","doc":"- loosen dependencies","ref":"changelog.html#changed"},{"type":"extras","title":"0.6.0 - 2024-11-27 - CHANGELOG","doc":"","ref":"changelog.html#0-6-0-2024-11-27"},{"type":"extras","title":"Changed - CHANGELOG","doc":"- Update Dependencies\n- Clean code\n- Support only for Elixir 1.12 and above","ref":"changelog.html#changed"},{"type":"extras","title":"0.5.0 - 2018-12-31 - CHANGELOG","doc":"Initial release.","ref":"changelog.html#0-5-0-2018-12-31"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.36.1"}}